"""Internationalization module for Hyperliquid CLI."""

import json
import os
from functools import lru_cache
from pathlib import Path
from typing import Optional

# Locales directory
LOCALES_DIR = Path(__file__).parent.parent / "locales"

# Supported languages
SUPPORTED_LANGUAGES = ["en", "zh-CN"]

# Default language
DEFAULT_LANGUAGE = "en"


@lru_cache(maxsize=10)
def load_translations(lang: str) -> dict:
    """Load translations for a language.

    Args:
        lang: Language code (e.g., "en", "zh-CN")

    Returns:
        Translation dictionary
    """
    path = LOCALES_DIR / f"{lang}.json"
    if path.exists():
        try:
            return json.loads(path.read_text(encoding="utf-8"))
        except json.JSONDecodeError:
            pass
    return {}


def get_nested_value(data: dict, key: str) -> str | dict:
    """Get nested value from dictionary using dot notation.

    Args:
        data: Dictionary to search
        key: Dot-separated key (e.g., "account.title")

    Returns:
        Value or empty dict if not found
    """
    keys = key.split(".")
    value = data
    for k in keys:
        if isinstance(value, dict):
            value = value.get(k, {})
        else:
            return key
    return value if isinstance(value, str) else key


def t(key: str, lang: Optional[str] = None, **kwargs) -> str:
    """Translate a key with optional formatting.

    Args:
        key: Translation key (e.g., "account.total_value")
        lang: Language code (defaults to HL_LANGUAGE env or "en")
        **kwargs: Format variables

    Returns:
        Translated and formatted string

    Example:
        >>> t("account.value", value=100.50)
        'Account Value: $100.50'
    """
    # Get language from parameter, env var, or default
    if lang is None:
        lang = os.getenv("HL_LANGUAGE", DEFAULT_LANGUAGE)

    # Fallback to en if language not supported
    if lang not in SUPPORTED_LANGUAGES:
        lang = DEFAULT_LANGUAGE

    translations = load_translations(lang)
    text = get_nested_value(translations, key)

    # Fallback to English if key not found
    if text == key and lang != "en":
        en_translations = load_translations("en")
        en_text = get_nested_value(en_translations, key)
        if en_text != key:
            text = en_text

    # Format with variables if provided
    if kwargs and text != key:
        try:
            return text.format(**kwargs)
        except (KeyError, ValueError):
            return text

    return text


def get_available_languages() -> list[str]:
    """Get list of available languages."""
    return SUPPORTED_LANGUAGES.copy()


def detect_language() -> str:
    """Detect system language.

    Returns:
        Detected language code or default
    """
    # Check environment variable first
    env_lang = os.getenv("HL_LANGUAGE")
    if env_lang and env_lang in SUPPORTED_LANGUAGES:
        return env_lang

    # Try to detect from system locale
    import locale

    try:
        system_locale = locale.getdefaultlocale()[0]
        if system_locale:
            # Map locale to supported language
            if system_locale.startswith("zh_CN") or system_locale.startswith("zh-CN"):
                return "zh-CN"
            elif system_locale.startswith("zh_TW") or system_locale.startswith("zh-TW"):
                return "zh-CN"  # Use simplified Chinese for now
    except Exception:
        pass

    return DEFAULT_LANGUAGE



"""Hyperliquid CLI - Command line interface for Hyperliquid trading."""

import json
import logging
import sys
from datetime import datetime, timezone
from pathlib import Path
from typing import Optional

import click
from rich.console import Console
from rich.logging import RichHandler
from rich.table import Table
from rich.tree import Tree

from .config import load_config, setup_config_interactive
from .hyperliquid_client import HyperliquidClient
from .i18n import t, detect_language

console = Console()


def get_timezone_str() -> str:
    """Get current timezone offset string."""
    offset = datetime.now().astimezone().utcoffset()
    if offset:
        hours = int(offset.total_seconds() / 3600)
        return f"UTC{hours:+d}"
    return "UTC"


def format_timestamp(ts_ms: int) -> str:
    """Format millisecond timestamp to local time."""
    dt = datetime.fromtimestamp(ts_ms / 1000)
    return dt.strftime("%Y-%m-%d %H:%M")


def setup_logging(log_level: str = "INFO"):
    """Setup logging with rich handler."""
    logging.basicConfig(
        level=getattr(logging, log_level.upper()),
        format="%(message)s",
        datefmt="[%X]",
        handlers=[RichHandler(console=console, rich_tracebacks=True)],
    )


def get_client(env_file: Optional[str] = None, require_auth: bool = True) -> HyperliquidClient:
    """Get configured client.

    Args:
        env_file: Path to .env file
        require_auth: If True, requires secret_key for trading operations
    """
    config = load_config(env_file)
    errors = config.validate(require_auth=require_auth)
    if errors:
        console.print("[red]Configuration errors:[/red]")
        for error in errors:
            console.print(f"  - {error}")
        console.print("\n[yellow]Run 'hl init' to set up configuration.[/yellow]")
        sys.exit(1)
    return HyperliquidClient(config)


def get_info_client(network: str = "testnet") -> HyperliquidClient:
    """Get read-only client for market data queries."""
    from scripts.config import HyperliquidConfig
    config = HyperliquidConfig(network=network)
    return HyperliquidClient(config)


@click.group()
@click.option("--env-file", "-e", help="Path to .env file")
@click.option("--verbose", "-v", is_flag=True, help="Enable verbose logging")
@click.option("--json", "output_json", "-j", is_flag=True, help="Output raw JSON")
@click.option("--lang", "-l", help="Language (en/zh-CN)")
@click.pass_context
def cli(ctx: click.Context, env_file: Optional[str], verbose: bool, output_json: bool, lang: Optional[str]):
    """Hyperliquid CLI - Trade crypto with ease."""
    ctx.ensure_object(dict)
    ctx.obj["env_file"] = env_file
    ctx.obj["output_json"] = output_json
    ctx.obj["lang"] = lang or detect_language()

    log_level = "DEBUG" if verbose else "INFO"
    setup_logging(log_level)


def output_result(ctx: click.Context, data: any, table_func=None, **kwargs):
    """Output result as JSON or formatted table.

    Args:
        ctx: Click context
        data: Data to output (will be JSON serialized if --json)
        table_func: Optional function to render table (called if not --json)
        **kwargs: Arguments passed to table_func
    """
    if ctx.obj.get("output_json"):
        console.print_json(json.dumps(data, default=str))
    elif table_func:
        table_func(**kwargs)
    else:
        console.print(data)


@cli.command()
def init():
    """Initialize configuration interactively."""
    setup_config_interactive()


# ==================== Market Data Commands ====================

@cli.group()
def market():
    """Market data commands."""
    pass


@market.command(name="prices")
@click.argument("coins", nargs=-1, required=False)
@click.option("--network", "-n", default="testnet", help="Network (mainnet/testnet)")
@click.pass_context
def market_prices(ctx, coins: tuple, network: str):
    """Get market prices. Optional COINS to filter (space-separated)."""
    client = get_info_client(network)
    try:
        prices = client.get_all_mids()

        if ctx.obj.get("output_json"):
            if coins:
                coins = [c.upper() for c in coins]
                prices = {c: prices.get(c) for c in coins}
            output_result(ctx, prices)
            return

        if coins:
            coins = [c.upper() for c in coins]
            table = Table(title="Market Prices")
            table.add_column("Asset", style="cyan")
            table.add_column("Price", justify="right", style="green")
            for c in coins:
                if c in prices:
                    table.add_row(c, f"${float(prices[c]):,.2f}")
                else:
                    table.add_row(c, "[red]Not found[/red]")
            console.print(table)
            return

        table = Table(title="Market Prices")
        table.add_column("Asset", style="cyan")
        table.add_column("Price", justify="right", style="green")

        for c, price in sorted(prices.items()):
            table.add_row(c, f"${float(price):,.2f}")

        console.print(table)
    finally:
        client.close()


@market.command(name="book")
@click.argument("coin")
@click.option("--depth", "-d", default=10, help="Number of levels to show")
@click.option("--network", "-n", default="testnet", help="Network (mainnet/testnet)")
@click.pass_context
def market_book(ctx, coin: str, depth: int, network: str):
    """Get order book for a coin."""
    client = get_info_client(network)
    try:
        book = client.get_l2_book(coin.upper())

        if ctx.obj.get("output_json"):
            output_result(ctx, book)
            return

        table = Table(title=f"Order Book - {coin.upper()}")
        table.add_column("Bid Price", justify="right", style="green")
        table.add_column("Bid Size", justify="right")
        table.add_column("Ask Price", justify="right", style="red")
        table.add_column("Ask Size", justify="right")

        bids = book.get("levels", [[]])[0][:depth]
        asks = book.get("levels", [[], []])[1][:depth]

        for i in range(max(len(bids), len(asks))):
            bid_px = bids[i]["px"] if i < len(bids) else ""
            bid_sz = bids[i]["sz"] if i < len(bids) else ""
            ask_px = asks[i]["px"] if i < len(asks) else ""
            ask_sz = asks[i]["sz"] if i < len(asks) else ""
            table.add_row(bid_px, bid_sz, ask_px, ask_sz)

        console.print(table)
    finally:
        client.close()


@market.command(name="context")
@click.argument("coins", nargs=-1, required=False)
@click.option("--network", "-n", default="testnet", help="Network (mainnet/testnet)")
@click.pass_context
def market_context(ctx, coins: tuple, network: str):
    """Get asset contexts (trading parameters). Optional COINS to filter (space-separated)."""
    client = get_info_client(network)
    try:
        contexts = client.get_asset_contexts()
        universe = contexts.get("universe", [])

        if ctx.obj.get("output_json"):
            if coins:
                coins = [c.upper() for c in coins]
                universe = [a for a in universe if a.get("name") in coins]
            output_result(ctx, {"universe": universe})
            return

        if coins:
            coins = [c.upper() for c in coins]
            table = Table(title="Asset Contexts")
            table.add_column("Coin", style="cyan")
            table.add_column("Max Leverage")
            table.add_column("Size Decimals")
            for c in coins:
                asset = next((a for a in universe if a.get("name") == c), None)
                if asset:
                    table.add_row(
                        c,
                        str(asset.get("maxLeverage", "N/A")),
                        str(asset.get("szDecimals", "N/A")),
                    )
                else:
                    table.add_row(c, "[red]Not found[/red]", "")
            console.print(table)
            return

        table = Table(title="Asset Contexts")
        table.add_column("Coin", style="cyan")
        table.add_column("Max Leverage")
        table.add_column("Size Decimals")

        for asset in universe:
            table.add_row(
                asset.get("name", "N/A"),
                str(asset.get("maxLeverage", "N/A")),
                str(asset.get("szDecimals", "N/A")),
            )

        console.print(table)
    finally:
        client.close()


@market.command(name="candles")
@click.argument("coin")
@click.option("--interval", "-i", default="1h", help="Candle interval (1m, 5m, 15m, 1h, 4h, 1d, 1w, 1M, 3M, 6M)")
@click.option("--hours", "-H", default=None, type=int, help="Hours of history")
@click.option("--days", "-d", default=None, type=int, help="Days of history (alternative to -H)")
@click.option("--network", "-n", default="testnet", help="Network (mainnet/testnet)")
@click.option("--limit", "-l", default=50, help="Max number of candles to show")
@click.pass_context
def market_candles(ctx, coin: str, interval: str, hours: Optional[int], days: Optional[int], network: str, limit: int):
    """Get candle/OHLCV data for a coin.

    Native intervals: 1m, 5m, 15m, 1h, 4h, 1d
    Aggregated intervals (via 1d): 1w (week), 1M (month), 3M (quarter), 6M (half-year)

    Examples:
        uv run hl market candles BTC -i 1h -H 24
        uv run hl market candles ETH -i 1d -d 30 -n mainnet
        uv run hl market candles BTC -i 1w -d 365 -n mainnet
        uv run hl market candles BTC -i 1M -d 365 -n mainnet
        uv run hl market candles BTC -i 3M -d 365 -n mainnet
    """
    import time
    from datetime import datetime

    # Aggregated intervals need 1d data
    aggregated_intervals = {"1w": 7, "1M": 30, "3M": 90, "6M": 180}
    is_aggregated = interval in aggregated_intervals
    api_interval = "1d" if is_aggregated else interval

    # Calculate hours from --days or --hours (default 24h)
    if days is not None:
        total_hours = days * 24
    elif hours is not None:
        total_hours = hours
    else:
        total_hours = 24

    client = get_info_client(network)
    try:
        end_time = int(time.time() * 1000)
        start_time = end_time - (total_hours * 60 * 60 * 1000)

        candles = client.get_candle_snapshot(
            coin=coin.upper(),
            interval=api_interval,
            start_time=start_time,
            end_time=end_time,
        )

        if not candles:
            if ctx.obj.get("output_json"):
                output_result(ctx, {"coin": coin.upper(), "candles": []})
            else:
                console.print(f"[yellow]No candle data found for {coin.upper()}[/yellow]")
            return

        # Aggregate candles if needed
        if is_aggregated:
            candles = _aggregate_candles(candles, aggregated_intervals[interval])

        if ctx.obj.get("output_json"):
            output_result(ctx, {"coin": coin.upper(), "interval": interval, "candles": candles[-limit:]})
            return

        tz = get_timezone_str()
        table = Table(title=f"{coin.upper()} Candles ({interval}) [{tz}]")
        table.add_column("Time", style="cyan")
        table.add_column("Open", justify="right", style="green")
        table.add_column("High", justify="right", style="green")
        table.add_column("Low", justify="right", style="red")
        table.add_column("Close", justify="right", style="yellow")
        table.add_column("Volume", justify="right")

        for candle in candles[-limit:]:
            t = format_timestamp(candle.get("t", 0))
            o = float(candle.get("o", 0))
            h = float(candle.get("h", 0))
            l = float(candle.get("l", 0))
            c = float(candle.get("c", 0))
            v = float(candle.get("v", 0))

            # Color close based on direction
            close_str = f"${c:,.2f}" if c >= o else f"[red]${c:,.2f}[/red]"

            table.add_row(
                t,
                f"${o:,.2f}",
                f"${h:,.2f}",
                f"${l:,.2f}",
                close_str,
                f"{v:,.2f}",
            )

        console.print(table)
    finally:
        client.close()


def _aggregate_candles(candles: list[dict], days_per_candle: int) -> list[dict]:
    """Aggregate daily candles into larger periods."""
    if not candles:
        return []

    aggregated = []
    for i in range(0, len(candles), days_per_candle):
        chunk = candles[i:i + days_per_candle]
        if not chunk:
            continue

        agg = {
            "t": chunk[0]["t"],  # First candle's time
            "o": chunk[0]["o"],  # First candle's open
            "h": max(float(c["h"]) for c in chunk),  # Highest high
            "l": min(float(c["l"]) for c in chunk),  # Lowest low
            "c": chunk[-1]["c"],  # Last candle's close
            "v": sum(float(c["v"]) for c in chunk),  # Sum of volumes
        }
        aggregated.append(agg)

    return aggregated


@market.command(name="funding")
@click.argument("coin")
@click.option("--network", "-n", default="testnet", help="Network (mainnet/testnet)")
@click.option("--limit", "-l", default=10, help="Number of records to show")
@click.pass_context
def market_funding(ctx, coin: str, network: str, limit: int):
    """Get recent funding rate for a coin."""
    client = get_info_client(network)
    try:
        funding = client.get_recent_funding(coin.upper())

        if ctx.obj.get("output_json"):
            output_result(ctx, {"coin": coin.upper(), "funding": funding[:limit]})
            return

        table = Table(title=f"{coin.upper()} Funding History [{get_timezone_str()}]")
        table.add_column("Time", style="cyan")
        table.add_column("Funding Rate", justify="right")
        table.add_column("Premium", justify="right")

        for record in funding[:limit]:
            ts = format_timestamp(record["time"])
            rate = float(record["fundingRate"]) * 100
            premium = float(record["premium"]) * 100
            rate_str = f"{rate:+.4f}%"
            premium_str = f"{premium:+.4f}%"
            table.add_row(ts, rate_str, premium_str)

        console.print(table)
    finally:
        client.close()


# ==================== Account Commands ====================

@cli.group()
def account():
    """Account information commands."""
    pass

@account.command(name="state")
@click.option("--address", "-a", help="Address to query (defaults to configured)")
@click.pass_context
def account_state(ctx, address: Optional[str]):
    """Get account state (balance, positions)."""
    from datetime import datetime
    from rich.panel import Panel
    from rich.columns import Columns

    lang = ctx.obj.get("lang", "en")
    client = get_client(ctx.obj.get("env_file"))
    try:
        state = client.get_user_state(address)

        if ctx.obj.get("output_json"):
            output_result(ctx, state)
            return

        margin = state.get("marginSummary", {})
        account_value = float(margin.get("accountValue", 0))
        total_ntl_pos = float(margin.get("totalNtlPos", 0))
        total_raw_usd = float(margin.get("totalRawUsd", 0))
        total_margin_used = float(margin.get("totalMarginUsed", 0))
        withdrawable = float(state.get("withdrawable", 0))

        # Risk metrics
        margin_utilization = (total_margin_used / account_value * 100) if account_value > 0 else 0
        position_ratio = (total_ntl_pos / account_value * 100) if account_value > 0 else 0

        # Account Overview Panel
        overview_text = (
            f"[bold]{t('account.total_value', lang)}:[/bold] ${account_value:,.2f}\n"
            f"[bold]{t('account.position_value', lang)}:[/bold] ${total_ntl_pos:,.2f}\n"
            f"[bold]{t('account.available_margin', lang)}:[/bold] ${total_raw_usd:,.2f}\n"
            f"[bold]{t('account.margin_used', lang)}:[/bold] ${total_margin_used:,.2f}\n"
            f"[bold]{t('account.withdrawable', lang)}:[/bold] ${withdrawable:,.2f}"
        )
        overview_panel = Panel(overview_text, title=f"[bold cyan]{t('account.title', lang)}[/bold cyan]", border_style="cyan")
        console.print(Columns([overview_panel]))

        # Risk Metrics Panel
        risk_color = "green" if margin_utilization < 50 else "yellow" if margin_utilization < 80 else "red"
        leverage_str = f"{total_ntl_pos / total_margin_used:.1f}x" if total_margin_used > 0 else "N/A"
        risk_text = (
            f"[bold]{t('risk.margin_utilization', lang)}:[/bold] [{risk_color}]{margin_utilization:.2f}%[/{risk_color}]\n"
            f"[bold]{t('risk.position_ratio', lang)}:[/bold] {position_ratio:.2f}%\n"
            f"[bold]{t('risk.leverage', lang)}:[/bold] {leverage_str}"
        )
        risk_panel = Panel(risk_text, title=f"[bold yellow]{t('risk.title', lang)}[/bold yellow]", border_style="yellow")

        console.print(Columns([risk_panel]))

        # Positions Table
        positions = state.get("assetPositions", [])
        if positions:
            table = Table(title=f"[bold]{t('account.position_details', lang)}[/bold]", show_lines=True)
            table.add_column(t("position.asset", lang), style="cyan bold", width=4)
            table.add_column(t("position.amount", lang), justify="right", width=12)
            table.add_column(t("position.leverage", lang), justify="center", width=8)
            table.add_column(t("position.entry_price", lang), justify="right", width=12)
            table.add_column(t("position.current_value", lang), justify="right", width=12)
            table.add_column(t("position.unrealized_pnl", lang), justify="right", width=12)
            table.add_column(t("position.roe", lang), justify="right", width=8)
            table.add_column(t("position.liquidation_price", lang), justify="right", width=16)
            table.add_column(t("position.margin_ratio", lang), justify="right", width=12)

            total_pnl = 0
            for pos in positions:
                p = pos.get("position", {})
                szi = float(p.get("szi", 0))
                if szi == 0:
                    continue

                coin = p.get("coin", "N/A")
                entry_px = float(p.get("entryPx", 0))
                pos_val = float(p.get("positionValue", 0))
                pnl = float(p.get("unrealizedPnl", 0))
                total_pnl += pnl
                roe = float(p.get("returnOnEquity", 0)) * 100
                liq_px = float(p.get("liquidationPx", 0))
                margin_used = float(p.get("marginUsed", 0))
                lev = p.get("leverage", {})
                lev_val = lev.get("value", "N/A")
                lev_type = lev.get("type", "cross")[0].upper()  # I or C

                margin_ratio = (margin_used / account_value * 100) if account_value > 0 else 0
                pnl_color = "green" if pnl >= 0 else "red"
                roe_color = "green" if roe >= 0 else "red"

                table.add_row(
                    coin,
                    f"{szi:+.4f}",
                    f"{lev_val}x {lev_type}",
                    f"${entry_px:,.2f}",
                    f"${pos_val:,.2f}",
                    f"[{pnl_color}]${pnl:+,.2f}[/{pnl_color}]",
                    f"[{roe_color}]{roe:+.2f}%[/{roe_color}]",
                    f"${liq_px:,.2f}",
                    f"{margin_ratio:.1f}%"
                )

            console.print(table)

            # Total PNL summary
            total_pnl_color = "green" if total_pnl >= 0 else "red"
            console.print(f"\n[bold]{t('account.total_unrealized_pnl', lang)}:[/bold] [{total_pnl_color}]${total_pnl:+,.2f}[/{total_pnl_color}]")

        # Timestamp
        ts = state.get("time")
        if ts:
            dt = datetime.fromtimestamp(ts / 1000)
            console.print(f"\n[dim]{t('time.last_updated', lang)}: {dt.strftime('%Y-%m-%d %H:%M:%S')} ({get_timezone_str()})[/dim]")
    finally:
        client.close()


@account.command(name="orders")
@click.option("--address", "-a", help="Address to query")
@click.pass_context
def account_orders(ctx, address: Optional[str]):
    """Get open orders."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        orders = client.get_open_orders(address)

        if ctx.obj.get("output_json"):
            output_result(ctx, {"orders": orders})
            return

        if not orders:
            console.print("[yellow]No open orders[/yellow]")
            return

        table = Table(title="Open Orders")
        table.add_column("Coin", style="cyan")
        table.add_column("Side", style="green")
        table.add_column("Size", justify="right")
        table.add_column("Price", justify="right")
        table.add_column("Order ID")

        for order in orders:
            side = "[green]Buy[/green]" if order.get("side") == "B" else "[red]Sell[/red]"
            table.add_row(
                order.get("coin", "N/A"),
                side,
                str(order.get("sz", "N/A")),
                f"${order.get('limitPx', 'N/A')}",
                str(order.get("oid", "N/A")),
            )

        console.print(table)
    finally:
        client.close()


@account.command(name="fills")
@click.option("--address", "-a", help="Address to query")
@click.option("--limit", "-l", default=20, help="Number of fills to show")
@click.pass_context
def account_fills(ctx, address: Optional[str], limit: int):
    """Get recent fills."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        fills = client.get_user_fills(address)[:limit]

        if ctx.obj.get("output_json"):
            output_result(ctx, {"fills": fills})
            return

        if not fills:
            console.print("[yellow]No fills found[/yellow]")
            return

        table = Table(title=f"Recent Fills [{get_timezone_str()}]")
        table.add_column("Time")
        table.add_column("Coin", style="cyan")
        table.add_column("Side", style="green")
        table.add_column("Size", justify="right")
        table.add_column("Price", justify="right")
        table.add_column("PNL", justify="right")

        for fill in fills:
            side = "[green]Buy[/green]" if fill.get("side") == "B" else "[red]Sell[/red]"
            table.add_row(
                format_timestamp(fill.get("time", 0)),
                fill.get("coin", "N/A"),
                side,
                str(fill.get("sz", "N/A")),
                f"${fill.get('px', 'N/A')}",
                f"${float(fill.get('closedPnl', 0)):,.2f}",
            )

        console.print(table)
    finally:
        client.close()


@account.command(name="funding")
@click.option("--address", "-a", help="Address to query")
@click.option("--limit", "-l", default=20, help="Number of records to show")
@click.pass_context
def account_funding(ctx, address: Optional[str], limit: int):
    """Get funding payment history."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        funding = client.get_account_funding_history(address)[:limit]

        if ctx.obj.get("output_json"):
            output_result(ctx, {"funding": funding})
            return

        if not funding:
            console.print("[yellow]No funding history found[/yellow]")
            return

        table = Table(title=f"Funding History [{get_timezone_str()}]")
        table.add_column("Time")
        table.add_column("Coin", style="cyan")
        table.add_column("Payment", justify="right")

        for f in funding:
            payment = float(f.get("fundingPayment", 0))
            color = "green" if payment >= 0 else "red"
            table.add_row(
                format_timestamp(f.get("time", 0)),
                f.get("coin", "N/A"),
                f"[{color}]${payment:,.4f}[/{color}]",
            )

        console.print(table)
    finally:
        client.close()


@account.command(name="rewards")
@click.option("--address", "-a", help="Address to query")
@click.pass_context
def account_rewards(ctx, address: Optional[str]):
    """Get staking rewards history."""
    client = get_client(ctx.obj.get("env_file"), require_auth=False)
    try:
        rewards = client.get_staking_rewards(address)

        if ctx.obj.get("output_json"):
            output_result(ctx, {"rewards": rewards})
            return

        if not rewards:
            console.print("[yellow]No staking rewards found[/yellow]")
            return

        table = Table(title=f"Staking Rewards [{get_timezone_str()}]")
        table.add_column("Time")
        table.add_column("Source", style="cyan")
        table.add_column("Amount", justify="right", style="green")

        for reward in rewards:
            amount = float(reward.get("totalAmount", 0))
            table.add_row(
                format_timestamp(reward.get("time", 0)),
                reward.get("source", "N/A"),
                f"{amount:,.4f} HYPE",
            )

        console.print(table)
    finally:
        client.close()


# ==================== Staking Commands ====================

@cli.group()
def staking():
    """Staking commands for HYPE delegation."""
    pass


@staking.command(name="summary")
@click.option("--address", "-a", help="Address to query")
@click.pass_context
def staking_summary(ctx, address: Optional[str]):
    """Get staking summary."""
    client = get_client(ctx.obj.get("env_file"), require_auth=False)
    try:
        summary = client.get_staking_summary(address)

        if ctx.obj.get("output_json"):
            output_result(ctx, summary)
            return

        delegated = float(summary.get("delegated", 0))
        undelegated = float(summary.get("undelegated", 0))
        total_pending = float(summary.get("totalPendingWithdrawal", 0))
        n_pending = summary.get("nPendingWithdrawals", 0)

        table = Table(title="Staking Summary")
        table.add_column("Metric", style="cyan")
        table.add_column("Value", justify="right", style="green")
        table.add_row("Delegated", f"{delegated:,.4f} HYPE")
        table.add_row("Undelegated (Pending)", f"{undelegated:,.4f} HYPE")
        table.add_row("Total Pending Withdrawal", f"{total_pending:,.4f} HYPE")
        table.add_row("Pending Withdrawals Count", str(n_pending))

        console.print(table)
    finally:
        client.close()


@staking.command(name="validators")
@click.option("--limit", "-l", default=30, help="Number of validators to show")
@click.option("--sort", "sort_by", type=click.Choice(["stake", "commission", "name"]), default="stake", help="Sort by field")
@click.pass_context
def staking_validators(ctx, limit: int, sort_by: str):
    """Get list of all validators."""
    client = get_client(ctx.obj.get("env_file"), require_auth=False)
    try:
        validators = client.get_validators()

        if ctx.obj.get("output_json"):
            output_result(ctx, {"validators": validators[:limit]})
            return

        if not validators:
            console.print("[yellow]No validators found[/yellow]")
            return

        # Sort validators
        if sort_by == "stake":
            validators = sorted(validators, key=lambda x: float(x.get("stake", 0)), reverse=True)
        elif sort_by == "commission":
            validators = sorted(validators, key=lambda x: float(x.get("commission", 0)))
        else:  # name
            validators = sorted(validators, key=lambda x: x.get("name", "").lower())

        table = Table(title=f"Validators (Top {min(limit, len(validators))})")
        table.add_column("#", justify="right", style="dim")
        table.add_column("Name", style="cyan")
        table.add_column("Address", style="dim")
        table.add_column("Stake", justify="right", style="green")
        table.add_column("Commission", justify="right")
        table.add_column("Delegations", justify="right")

        for i, v in enumerate(validators[:limit]):
            stake = float(v.get("stake", 0))
            commission = float(v.get("commission", 0)) * 100
            n_delegations = v.get("nDelegations", 0)
            address = v.get("validator", "N/A")
            address_short = address[:10] + "..." if len(address) > 10 else address

            table.add_row(
                str(i + 1),
                v.get("name", "Unknown"),
                address_short,
                f"{stake:,.2f} HYPE",
                f"{commission:.1f}%",
                str(n_delegations),
            )

        console.print(table)
        console.print(f"\n[dim]Total validators: {len(validators)}[/dim]")
        console.print(f"[dim]Use --sort name|stake|commission to change sort order[/dim]")
    finally:
        client.close()


@staking.command(name="delegations")
@click.option("--address", "-a", help="Address to query")
@click.pass_context
def staking_delegations(ctx, address: Optional[str]):
    """Get staking delegations."""
    client = get_client(ctx.obj.get("env_file"), require_auth=False)
    try:
        delegations = client.get_staking_delegations(address)

        if ctx.obj.get("output_json"):
            output_result(ctx, {"delegations": delegations})
            return

        if not delegations:
            console.print("[yellow]No staking delegations found[/yellow]")
            return

        table = Table(title="Staking Delegations")
        table.add_column("Validator", style="cyan")
        table.add_column("Amount", justify="right", style="green")
        table.add_column("Locked Until")

        for delegation in delegations:
            amount = float(delegation.get("amount", 0))
            locked_until = delegation.get("lockedUntilTimestamp", 0)
            locked_str = format_timestamp(locked_until) if locked_until else "Unlocked"
            table.add_row(
                delegation.get("validator", "N/A")[:20] + "...",
                f"{amount:,.4f} HYPE",
                locked_str,
            )

        console.print(table)
    finally:
        client.close()


@staking.command(name="rewards")
@click.option("--address", "-a", help="Address to query")
@click.option("--limit", "-l", default=20, help="Number of records to show")
@click.pass_context
def staking_rewards(ctx, address: Optional[str], limit: int):
    """Get staking rewards history."""
    client = get_client(ctx.obj.get("env_file"), require_auth=False)
    try:
        rewards = client.get_staking_rewards(address)[:limit]

        if ctx.obj.get("output_json"):
            output_result(ctx, {"rewards": rewards})
            return

        if not rewards:
            console.print("[yellow]No staking rewards found[/yellow]")
            return

        table = Table(title=f"Staking Rewards [{get_timezone_str()}]")
        table.add_column("Time")
        table.add_column("Source", style="cyan")
        table.add_column("Amount", justify="right", style="green")

        for reward in rewards:
            amount = float(reward.get("totalAmount", 0))
            table.add_row(
                format_timestamp(reward.get("time", 0)),
                reward.get("source", "N/A"),
                f"{amount:,.6f} HYPE",
            )

        console.print(table)
    finally:
        client.close()


@staking.command(name="history")
@click.option("--address", "-a", help="Address to query")
@click.option("--limit", "-l", default=20, help="Number of records to show")
@click.pass_context
def staking_history(ctx, address: Optional[str], limit: int):
    """Get staking history (delegations/undelegations)."""
    client = get_client(ctx.obj.get("env_file"), require_auth=False)
    try:
        history = client.get_staking_history(address)[:limit]

        if ctx.obj.get("output_json"):
            output_result(ctx, {"history": history})
            return

        if not history:
            console.print("[yellow]No staking history found[/yellow]")
            return

        table = Table(title=f"Staking History [{get_timezone_str()}]")
        table.add_column("Time")
        table.add_column("Type", style="cyan")
        table.add_column("Validator")
        table.add_column("Amount", justify="right", style="green")
        table.add_column("Hash", style="dim")

        for event in history:
            event_type = event.get("type", "N/A")
            is_delegate = "delegate" in event_type.lower() and "undelegate" not in event_type.lower()
            type_str = "[green]Delegate[/green]" if is_delegate else "[red]Undelegate[/red]"

            amount = float(event.get("amount", 0))
            hash_val = event.get("hash", "N/A")
            hash_str = hash_val[:16] + "..." if hash_val and len(hash_val) > 16 else hash_val

            table.add_row(
                format_timestamp(event.get("time", 0)),
                type_str,
                event.get("validator", "N/A")[:16] + "...",
                f"{amount:,.4f} HYPE",
                hash_str,
            )

        console.print(table)
    finally:
        client.close()


@staking.command(name="delegate")
@click.argument("validator")
@click.argument("amount", type=float)
@click.confirmation_option("-y", "--yes", prompt="Confirm delegation?")
@click.pass_context
def staking_delegate(ctx, validator: str, amount: float):
    """Delegate HYPE tokens to a validator.

    VALIDATOR: Validator address (0x...)
    AMOUNT: Amount of HYPE to delegate
    """
    client = get_client(ctx.obj.get("env_file"))
    try:
        result = client.delegate(validator, amount)
        console.print(f"[green]Successfully delegated {amount} HYPE to {validator[:16]}...[/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


@staking.command(name="undelegate")
@click.argument("validator")
@click.argument("amount", type=float)
@click.confirmation_option("-y", "--yes", prompt="Confirm undelegation?")
@click.pass_context
def staking_undelegate(ctx, validator: str, amount: float):
    """Undelegate HYPE tokens from a validator.

    VALIDATOR: Validator address (0x...)
    AMOUNT: Amount of HYPE to undelegate
    """
    client = get_client(ctx.obj.get("env_file"))
    try:
        result = client.undelegate(validator, amount)
        console.print(f"[green]Successfully undelegated {amount} HYPE from {validator[:16]}...[/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


# ==================== Trading Commands ====================

@cli.group()
def trade():
    """Trading commands."""
    pass


@trade.command(name="buy")
@click.argument("coin")
@click.argument("size", type=float)
@click.option("--price", "-p", type=float, help="Limit price (omit for market order)")
@click.option("--cloid", help="Client order ID")
@click.confirmation_option("-y", "--yes", prompt="Confirm buy order?")
@click.pass_context
def trade_buy(ctx, coin: str, size: float, price: Optional[float], cloid: Optional[str]):
    """Place a buy order."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        if price:
            result = client.place_order(
                coin=coin.upper(),
                is_buy=True,
                sz=size,
                limit_px=price,
                order_type={"limit": {"tif": "Gtc"}},
                cloid=cloid,
            )
        else:
            result = client.place_market_order(
                coin=coin.upper(),
                is_buy=True,
                sz=size,
                cloid=cloid,
            )
        console.print("[green]Order placed successfully![/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


@trade.command(name="sell")
@click.argument("coin")
@click.argument("size", type=float)
@click.option("--price", "-p", type=float, help="Limit price (omit for market order)")
@click.option("--reduce-only", is_flag=True, help="Reduce-only order")
@click.option("--cloid", help="Client order ID")
@click.confirmation_option("-y", "--yes", prompt="Confirm sell order?")
@click.pass_context
def trade_sell(ctx, coin: str, size: float, price: Optional[float], reduce_only: bool, cloid: Optional[str]):
    """Place a sell order."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        if price:
            result = client.place_order(
                coin=coin.upper(),
                is_buy=False,
                sz=size,
                limit_px=price,
                order_type={"limit": {"tif": "Gtc"}},
                reduce_only=reduce_only,
                cloid=cloid,
            )
        else:
            result = client.place_market_order(
                coin=coin.upper(),
                is_buy=False,
                sz=size,
                reduce_only=reduce_only,
                cloid=cloid,
            )
        console.print("[green]Order placed successfully![/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


@trade.command(name="cancel")
@click.argument("coin")
@click.argument("order_id", type=int)
@click.confirmation_option("-y", "--yes", prompt="Confirm cancellation?")
@click.pass_context
def trade_cancel(ctx, coin: str, order_id: int):
    """Cancel an order by ID."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        result = client.cancel_order(coin.upper(), order_id)
        console.print("[green]Order cancelled![/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


@trade.command(name="cancel-all")
@click.option("--coin", "-c", help="Filter by coin")
@click.confirmation_option("-y", "--yes", prompt="Cancel all open orders?")
@click.pass_context
def trade_cancel_all(ctx, coin: Optional[str]):
    """Cancel all open orders."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        result = client.cancel_all_orders(coin.upper() if coin else None)
        console.print(f"[green]Cancelled {result['cancelled']} orders[/green]")
    finally:
        client.close()


@trade.command(name="modify")
@click.argument("coin")
@click.argument("order_id", type=int)
@click.argument("new_size", type=float)
@click.argument("new_price", type=float)
@click.confirmation_option("-y", "--yes", prompt="Confirm order modification?")
@click.pass_context
def trade_modify(ctx, coin: str, order_id: int, new_size: float, new_price: float):
    """Modify an existing order."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        result = client.modify_order(coin.upper(), order_id, new_size, new_price)
        console.print("[green]Order modified![/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


@trade.command(name="close")
@click.argument("coin")
@click.confirmation_option("-y", "--yes", prompt="Close position?")
@click.pass_context
def trade_close(ctx, coin: str):
    """Close position for a coin."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        result = client.close_position(coin.upper())
        console.print("[green]Position closed![/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


# ==================== Leverage Commands ====================

@cli.group()
def leverage():
    """Leverage and margin commands."""
    pass


@leverage.command(name="set")
@click.argument("coin")
@click.argument("value", type=int)
@click.option("--cross", is_flag=True, help="Use cross margin")
@click.confirmation_option("-y", "--yes", prompt="Update leverage?")
@click.pass_context
def leverage_set(ctx, coin: str, value: int, cross: bool):
    """Set leverage for a coin."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        result = client.update_leverage(coin.upper(), value, is_cross=cross)
        margin_type = "cross" if cross else "isolated"
        console.print(f"[green]Leverage updated to {value}x {margin_type}![/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


@leverage.command(name="margin")
@click.argument("coin")
@click.argument("amount", type=float)
@click.confirmation_option("-y", "--yes", prompt="Update margin?")
@click.pass_context
def leverage_margin(ctx, coin: str, amount: float):
    """Update isolated margin for a position."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        result = client.update_isolated_margin(coin.upper(), amount)
        action = "Added" if amount > 0 else "Removed"
        console.print(f"[green]{action} ${abs(amount)} margin![/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


# ==================== Transfer Commands ====================

@cli.group()
def transfer():
    """Transfer and withdrawal commands."""
    pass


@transfer.command(name="send")
@click.argument("destination")
@click.argument("amount", type=float)
@click.option("--token", default="USDC", help="Token to transfer")
@click.confirmation_option("-y", "--yes", prompt="Confirm transfer?")
@click.pass_context
def transfer_send(ctx, destination: str, amount: float, token: str):
    """Transfer tokens to another address."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        result = client.transfer(destination, amount, token)
        console.print(f"[green]Transferred {amount} {token}![/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


@transfer.command(name="withdraw")
@click.argument("destination")
@click.argument("amount", type=float)
@click.option("--token", default="USDC", help="Token to withdraw")
@click.confirmation_option("-y", "--yes", prompt="Confirm withdrawal?")
@click.pass_context
def transfer_withdraw(ctx, destination: str, amount: float, token: str):
    """Withdraw tokens to another chain."""
    client = get_client(ctx.obj.get("env_file"))
    try:
        result = client.withdraw(destination, amount, token)
        console.print(f"[green]Withdrew {amount} {token}![/green]")
        console.print_json(json.dumps(result))
    finally:
        client.close()


# ==================== Utility Commands ====================

@cli.command()
def version():
    """Show CLI version."""
    console.print("[bold cyan]Hyperliquid CLI[/bold cyan] v0.1.0")


def main():
    """Entry point."""
    cli()


if __name__ == "__main__":
    main()



"""Hyperliquid SDK client wrapper."""

import logging
from typing import Any, Optional

from hyperliquid.api import API
from hyperliquid.exchange import Exchange
from hyperliquid.info import Info
from hyperliquid.utils.types import Cloid

from scripts.config import HyperliquidConfig

logger = logging.getLogger(__name__)


class HyperliquidClient:
    """Client for Hyperliquid API operations."""

    def __init__(self, config: HyperliquidConfig):
        """Initialize client with configuration.

        Args:
            config: Hyperliquid configuration
        """
        self.config = config
        self._api: Optional[API] = None
        self._info: Optional[Info] = None
        self._exchange: Optional[Exchange] = None

    @property
    def api(self) -> API:
        """Get or create API instance for direct API calls."""
        if self._api is None:
            self._api = API(self.config.api_url)
            logger.debug(f"API client initialized: {self.config.api_url}")
        return self._api

    @property
    def info(self) -> Info:
        """Get or create Info instance."""
        if self._info is None:
            base_url = self.config.api_url
            self._info = Info(base_url, skip_ws=True)
            logger.debug(f"Info client initialized: {base_url}")
        return self._info

    @property
    def exchange(self) -> Exchange:
        """Get or create Exchange instance."""
        if self._exchange is None:
            base_url = self.config.api_url
            # Use API secret key if available, otherwise use main secret key
            secret_key = self.config.api_secret_key or self.config.secret_key
            if not secret_key:
                raise ValueError("Secret key required for trading operations")
            from eth_account import Account
            wallet = Account.from_key(secret_key)

            # Fetch and filter spot_meta to fix testnet API bug
            # (some token indices may be out of range)
            spot_meta = self._get_filtered_spot_meta()

            self._exchange = Exchange(
                wallet,
                base_url=base_url,
                account_address=self.config.account_address,
                spot_meta=spot_meta,
            )
            logger.debug(f"Exchange client initialized: {base_url}")
        return self._exchange

    def _get_filtered_spot_meta(self) -> dict:
        """Fetch spot_meta and filter out invalid entries."""
        resp = self.api.post("/info", {"type": "spotMeta"})
        spot_meta = resp

        # Filter out universe items with out-of-range token indices
        tokens_count = len(spot_meta.get("tokens", []))
        valid_universe = [
            item for item in spot_meta.get("universe", [])
            if all(idx < tokens_count for idx in item.get("tokens", []))
        ]
        spot_meta["universe"] = valid_universe
        return spot_meta

    def close(self):
        """Close all connections."""
        if self._info:
            self._info.disconnect_websocket()
            self._info = None
        self._exchange = None
        self._api = None

    # ==================== Info Methods (using direct API for stability) ====================

    def get_all_mids(self) -> dict[str, str]:
        """Get all market mid prices.

        Returns:
            Dictionary mapping coin to mid price
        """
        return self.api.post("/info", {"type": "allMids"})

    def get_user_state(self, address: Optional[str] = None) -> dict:
        """Get user state including balances and positions.

        Args:
            address: User address (defaults to configured address)

        Returns:
            User state data
        """
        addr = address or self.config.account_address
        return self.api.post("/info", {"type": "clearinghouseState", "user": addr})

    def get_user_fills(self, address: Optional[str] = None) -> list[dict]:
        """Get user's fill history.

        Args:
            address: User address (defaults to configured address)

        Returns:
            List of fill records
        """
        addr = address or self.config.account_address
        return self.api.post("/info", {"type": "userFills", "user": addr})

    def get_open_orders(self, address: Optional[str] = None) -> list[dict]:
        """Get user's open orders.

        Args:
            address: User address (defaults to configured address)

        Returns:
            List of open orders
        """
        addr = address or self.config.account_address
        return self.api.post("/info", {"type": "openOrders", "user": addr})

    def get_order_status(self, address: str, oid: int) -> dict:
        """Get status of a specific order.

        Args:
            address: User address
            oid: Order ID

        Returns:
            Order status data
        """
        return self.api.post("/info", {"type": "orderStatus", "user": address, "oid": oid})

    def get_asset_contexts(self) -> dict:
        """Get asset contexts (trading parameters).

        Returns:
            Asset context data with universe and tokens
        """
        return self.api.post("/info", {"type": "meta"})

    def get_spot_meta(self) -> dict:
        """Get spot market metadata.

        Returns:
            Spot metadata with tokens and universe
        """
        return self.api.post("/info", {"type": "spotMeta"})

    def get_candle_snapshot(
        self, coin: str, interval: str, start_time: int, end_time: int
    ) -> list[dict]:
        """Get candle/OHLCV data.

        Args:
            coin: Asset symbol (e.g., "BTC")
            interval: Time interval (e.g., "1h", "1d")
            start_time: Start timestamp (ms)
            end_time: End timestamp (ms)

        Returns:
            List of candle data
        """
        return self.api.post("/info", {
            "type": "candleSnapshot",
            "req": {
                "coin": coin,
                "interval": interval,
                "startTime": start_time,
                "endTime": end_time,
            }
        })

    def get_l2_book(self, coin: str) -> dict:
        """Get L2 order book.

        Args:
            coin: Asset symbol

        Returns:
            Order book data with levels [[bids], [asks]]
        """
        return self.api.post("/info", {"type": "l2Book", "coin": coin})

    def get_funding_history(
        self, coin: str, start_time: int, end_time: Optional[int] = None
    ) -> list[dict]:
        """Get funding rate history.

        Args:
            coin: Asset symbol
            start_time: Start timestamp (ms)
            end_time: End timestamp (ms), defaults to now

        Returns:
            List of funding rate records
        """
        req = {"type": "fundingHistory", "coin": coin, "startTime": start_time}
        if end_time:
            req["endTime"] = end_time
        return self.api.post("/info", req)

    def get_recent_funding(self, coin: str, hours: int = 24) -> list[dict]:
        """Get recent funding rate history.

        Args:
            coin: Asset symbol
            hours: Number of hours of history (default 24)

        Returns:
            Recent funding rate data
        """
        import time
        start_time = int(time.time() * 1000) - (hours * 60 * 60 * 1000)
        return self.api.post("/info", {"type": "fundingHistory", "coin": coin, "startTime": start_time})

    def get_account_funding_history(
        self, address: Optional[str] = None
    ) -> list[dict]:
        """Get account funding payment history.

        Args:
            address: User address (defaults to configured address)

        Returns:
            List of funding payments
        """
        addr = address or self.config.account_address
        return self.api.post("/info", {"type": "userFunding", "user": addr})

    def get_deposits(self, address: Optional[str] = None) -> list[dict]:
        """Get deposit history.

        Args:
            address: User address (defaults to configured address)

        Returns:
            List of deposit records
        """
        addr = address or self.config.account_address
        return self.api.post("/info", {"type": "userDeposits", "user": addr})

    def get_withdrawals(self, address: Optional[str] = None) -> list[dict]:
        """Get withdrawal history.

        Args:
            address: User address (defaults to configured address)

        Returns:
            List of withdrawal records
        """
        addr = address or self.config.account_address
        return self.api.post("/info", {"type": "userWithdrawals", "user": addr})

    # ==================== Trading Methods ====================

    def place_order(
        self,
        coin: str,
        is_buy: bool,
        sz: float,
        limit_px: float,
        order_type: dict = None,
        reduce_only: bool = False,
        cloid: Optional[str] = None,
    ) -> dict:
        """Place a new order.

        Args:
            coin: Asset symbol (e.g., "BTC")
            is_buy: True for buy, False for sell
            sz: Order size (in coins)
            limit_px: Limit price
            order_type: SDK format, e.g. {"limit": {"tif": "Gtc"}}
            reduce_only: Whether order is reduce-only
            cloid: Optional client order ID

        Returns:
            Order result
        """
        if order_type is None:
            order_type = {"limit": {"tif": "Gtc"}}
        cloid_obj = Cloid.from_str(cloid) if cloid else None

        order_result = self.exchange.order(
            name=coin,
            is_buy=is_buy,
            sz=sz,
            limit_px=limit_px,
            order_type=order_type,
            reduce_only=reduce_only,
            cloid=cloid_obj,
        )
        logger.info(f"Order placed: {order_result}")
        return order_result

    def place_market_order(
        self,
        coin: str,
        is_buy: bool,
        sz: float,
        reduce_only: bool = False,
        cloid: Optional[str] = None,
    ) -> dict:
        """Place a market order using IOC with order book pricing.

        Uses order book best bid/ask for more accurate pricing.

        Args:
            coin: Asset symbol
            is_buy: True for buy, False for sell
            sz: Order size
            reduce_only: Whether order is reduce-only
            cloid: Optional client order ID

        Returns:
            Order result
        """
        # Get order book for best bid/ask prices
        book = self.get_l2_book(coin)
        levels = book.get("levels", [[], []])
        bids = levels[0] if len(levels) > 0 else []
        asks = levels[1] if len(levels) > 1 else []

        if is_buy:
            # For buy, use best ask price + small buffer
            if not asks:
                raise ValueError(f"No asks available for {coin}")
            best_ask = float(asks[0]["px"])
            limit_px = int(best_ask) + 1  # Just 1 tick above ask
            ref_price = best_ask
        else:
            # For sell, use best bid price - small buffer
            if not bids:
                raise ValueError(f"No bids available for {coin}")
            best_bid = float(bids[0]["px"])
            limit_px = int(best_bid) - 1  # Just 1 tick below bid
            if limit_px <= 0:
                limit_px = 1  # Safety floor
            ref_price = best_bid

        logger.info(f"Market order: {coin} {'BUY' if is_buy else 'SELL'} {sz} @ ~${ref_price:.2f} (limit: {limit_px})")

        return self.place_order(
            coin=coin,
            is_buy=is_buy,
            sz=sz,
            limit_px=limit_px,
            order_type={"limit": {"tif": "Ioc"}},
            reduce_only=reduce_only,
            cloid=cloid,
        )

    def cancel_order(self, coin: str, oid: int) -> dict:
        """Cancel an order by ID.

        Args:
            coin: Asset symbol
            oid: Order ID

        Returns:
            Cancel result
        """
        result = self.exchange.cancel(coin, oid)
        logger.info(f"Order cancelled: {result}")
        return result

    def cancel_all_orders(self, coin: Optional[str] = None) -> dict:
        """Cancel all open orders.

        Args:
            coin: Optional coin to filter by

        Returns:
            Cancel result
        """
        # Get open orders first
        open_orders = self.get_open_orders()

        if coin:
            open_orders = [o for o in open_orders if o.get("coin") == coin]

        results = []
        for order in open_orders:
            coin_symbol = order.get("coin")
            oid = order.get("oid")
            if coin_symbol and oid:
                result = self.cancel_order(coin_symbol, oid)
                results.append(result)

        return {"cancelled": len(results), "results": results}

    def modify_order(
        self,
        coin: str,
        oid: int,
        new_sz: float,
        new_limit_px: float,
    ) -> dict:
        """Modify an existing order.

        Args:
            coin: Asset symbol
            oid: Order ID to modify
            new_sz: New size
            new_limit_px: New limit price

        Returns:
            Modify result
        """
        # 查询订单信息获取 is_buy
        open_orders = self.get_open_orders()
        target_order = None
        for order in open_orders:
            if order.get("oid") == oid:
                target_order = order
                break

        if not target_order:
            raise ValueError(f"Order {oid} not found in open orders")

        # side: "B" = Buy, "A" = Ask/Sell
        is_buy = target_order.get("side") == "B"

        # 使用默认的限价单类型
        order_type = {"limit": {"tif": "Gtc"}}

        result = self.exchange.modify_order(
            oid=oid,
            name=coin,
            is_buy=is_buy,
            sz=new_sz,
            limit_px=new_limit_px,
            order_type=order_type,
        )
        logger.info(f"Order modified: {result}")
        return result

    # ==================== Spot Trading ====================

    def place_spot_order(
        self,
        coin: str,
        is_buy: bool,
        sz: float,
        limit_px: float,
        order_type: str = "Limit",
    ) -> dict:
        """Place a spot order.

        Args:
            coin: Spot asset symbol
            is_buy: True for buy, False for sell
            sz: Order size
            limit_px: Limit price
            order_type: Order type

        Returns:
            Order result
        """
        order_result = self.exchange.spot_order(
            coin=coin,
            is_buy=is_buy,
            sz=sz,
            limit_px=limit_px,
            order_type=order_type,
        )
        logger.info(f"Spot order placed: {order_result}")
        return order_result

    def get_spot_balances(self, address: Optional[str] = None) -> list[dict]:
        """Get spot token balances.

        Args:
            address: User address (defaults to configured address)

        Returns:
            List of spot balances
        """
        addr = address or self.config.account_address
        user_state = self.get_user_state(addr)
        return user_state.get("balances", [])

    # ==================== Transfer/Withdraw ====================

    def transfer(self, destination: str, amount: float, token: str = "USDC") -> dict:
        """Transfer tokens to another address.

        Args:
            destination: Destination address
            amount: Amount to transfer
            token: Token symbol

        Returns:
            Transfer result
        """
        result = self.exchange.transfer(destination, amount, token)
        logger.info(f"Transfer executed: {result}")
        return result

    def withdraw(self, destination: str, amount: float, token: str = "USDC") -> dict:
        """Withdraw tokens to another chain.

        Args:
            destination: Destination address
            amount: Amount to withdraw
            token: Token symbol

        Returns:
            Withdrawal result
        """
        result = self.exchange.withdraw(destination, amount, token)
        logger.info(f"Withdrawal executed: {result}")
        return result

    # ==================== Leverage/Position ====================

    def update_leverage(self, coin: str, leverage: int, is_cross: bool = False) -> dict:
        """Update leverage for a coin.

        Args:
            coin: Asset symbol
            leverage: Leverage value (1-50)
            is_cross: True for cross margin, False for isolated

        Returns:
            Update result
        """
        result = self.exchange.update_leverage(leverage, coin, is_cross)
        logger.info(f"Leverage updated: {result}")
        return result

    def update_isolated_margin(self, coin: str, amount: float) -> dict:
        """Update isolated margin for a position.

        Args:
            coin: Asset symbol
            amount: Amount to add/remove

        Returns:
            Update result
        """
        result = self.exchange.update_isolated_margin(amount, coin)
        logger.info(f"Isolated margin updated: {result}")
        return result

    def close_position(self, coin: str) -> dict:
        """Close a position for a coin using IOC market order.

        Args:
            coin: Asset symbol

        Returns:
            Close result
        """
        # Get current position to determine size and direction
        user_state = self.get_user_state()

        # Handle both dict and list responses
        if isinstance(user_state, list):
            positions = user_state
        elif isinstance(user_state, dict):
            # Check for nested structure
            asset_positions = user_state.get("assetPositions")
            if isinstance(asset_positions, dict):
                positions = asset_positions.get("values", [])
            elif isinstance(asset_positions, list):
                positions = asset_positions
            else:
                positions = []
        else:
            positions = []

        position = None
        for pos in positions:
            pos_data = pos.get("position", pos)
            if pos_data.get("coin") == coin:
                position = pos_data
                break

        if not position:
            return {"status": "error", "message": f"No position found for {coin}"}

        szi = float(position.get("szi", 0))
        if szi == 0:
            return {"status": "error", "message": f"Position for {coin} is already closed"}

        # Close position: sell if long, buy if short (reduce-only)
        result = self.place_market_order(
            coin=coin,
            is_buy=(szi < 0),  # Buy to close short, sell to close long
            sz=abs(szi),
            reduce_only=True,
        )
        logger.info(f"Position closed: {result}")
        return result

    # ==================== Staking Methods ====================

    def get_validators(self) -> list[dict]:
        """Get list of all validators.

        Returns:
            List of validators with name, address, commission, stake info
        """
        return self.api.post("/info", {"type": "validatorSummaries"})

    def get_staking_summary(self, address: Optional[str] = None) -> dict:
        """Get staking summary for an address.

        Args:
            address: User address (defaults to configured address)

        Returns:
            Staking summary with delegated, undelegated amounts
        """
        addr = address or self.config.account_address
        return self.api.post("/info", {"type": "delegatorSummary", "user": addr})

    def get_staking_delegations(self, address: Optional[str] = None) -> list[dict]:
        """Get staking delegations for an address.

        Args:
            address: User address (defaults to configured address)

        Returns:
            List of delegations with validator, amount, locked_until_timestamp
        """
        addr = address or self.config.account_address
        return self.api.post("/info", {"type": "delegations", "user": addr})

    def get_staking_rewards(self, address: Optional[str] = None) -> list[dict]:
        """Get staking rewards history for an address.

        Args:
            address: User address (defaults to configured address)

        Returns:
            List of rewards with time, source, total_amount
        """
        addr = address or self.config.account_address
        return self.api.post("/info", {"type": "delegatorRewards", "user": addr})

    def get_staking_history(self, address: Optional[str] = None) -> list[dict]:
        """Get comprehensive staking history for an address.

        Args:
            address: User address (defaults to configured address)

        Returns:
            List of staking history events
        """
        addr = address or self.config.account_address
        return self.api.post("/info", {"type": "delegatorHistory", "user": addr})

    def delegate(self, validator: str, amount: float) -> dict:
        """Delegate tokens to a validator.

        Args:
            validator: Validator address
            amount: Amount to delegate (in HYPE)

        Returns:
            Delegation result
        """
        # Convert amount to wei (HYPE has 18 decimals)
        wei = int(amount * 10**18)
        result = self.exchange.token_delegate(validator, wei, is_undelegate=False)
        logger.info(f"Delegated {amount} HYPE to {validator}: {result}")
        return result

    def undelegate(self, validator: str, amount: float) -> dict:
        """Undelegate tokens from a validator.

        Args:
            validator: Validator address
            amount: Amount to undelegate (in HYPE)

        Returns:
            Undelegation result
        """
        # Convert amount to wei (HYPE has 18 decimals)
        wei = int(amount * 10**18)
        result = self.exchange.token_delegate(validator, wei, is_undelegate=True)
        logger.info(f"Undelegated {amount} HYPE from {validator}: {result}")
        return result



"""Configuration management for Hyperliquid CLI."""

import os
from dataclasses import dataclass
from pathlib import Path
from typing import Optional

from dotenv import load_dotenv


@dataclass
class HyperliquidConfig:
    """Hyperliquid configuration."""

    network: str = "testnet"
    account_address: Optional[str] = None
    secret_key: Optional[str] = None
    api_secret_key: Optional[str] = None
    log_level: str = "INFO"
    language: str = "en"

    @property
    def is_mainnet(self) -> bool:
        """Check if using mainnet."""
        return self.network.lower() == "mainnet"

    @property
    def api_url(self) -> str:
        """Get API URL based on network."""
        if self.is_mainnet:
            return "https://api.hyperliquid.xyz"
        return "https://api.hyperliquid-testnet.xyz"

    def validate(self, require_auth: bool = True) -> list[str]:
        """Validate configuration and return list of errors.

        Args:
            require_auth: If True, requires secret_key for validation
        """
        errors = []
        if not self.account_address:
            errors.append("HL_ACCOUNT_ADDRESS is required")
        if require_auth and not self.secret_key and not self.api_secret_key:
            errors.append("HL_SECRET_KEY is required for trading operations")
        return errors


def load_config(env_file: Optional[str] = None) -> HyperliquidConfig:
    """Load configuration from environment variables.

    Args:
        env_file: Optional path to .env file

    Returns:
        HyperliquidConfig instance
    """
    # Load from .env file if provided or exists
    if env_file:
        load_dotenv(env_file)
    else:
        # Try to find .env in current directory and project root
        cwd_env = Path(".env")
        if cwd_env.exists():
            load_dotenv(cwd_env)
        else:
            project_root = Path(__file__).parent.parent
            root_env = project_root / ".env"
            if root_env.exists():
                load_dotenv(root_env)

    return HyperliquidConfig(
        network=os.getenv("HL_NETWORK", "testnet"),
        account_address=os.getenv("HL_ACCOUNT_ADDRESS"),
        secret_key=os.getenv("HL_SECRET_KEY"),
        api_secret_key=os.getenv("HL_API_SECRET_KEY"),
        log_level=os.getenv("HL_LOG_LEVEL", "INFO"),
        language=os.getenv("HL_LANGUAGE", "en"),
    )


def setup_config_interactive() -> HyperliquidConfig:
    """Interactive configuration setup."""
    print("Hyperliquid CLI Configuration Setup")
    print("=" * 40)

    network = input("Network (mainnet/testnet) [testnet]: ").strip() or "testnet"
    account_address = input("Your wallet address (0x...): ").strip()
    secret_key = input("Your private key (0x...): ").strip()
    api_secret_key = input("API wallet private key (optional): ").strip() or None
    log_level = input("Log level (DEBUG/INFO/WARNING/ERROR) [INFO]: ").strip() or "INFO"
    language = input("Language (en/zh-CN) [en]: ").strip() or "en"

    config = HyperliquidConfig(
        network=network,
        account_address=account_address,
        secret_key=secret_key,
        api_secret_key=api_secret_key,
        log_level=log_level,
        language=language,
    )

    # Save to .env file
    env_content = f"""# Hyperliquid API Configuration
HL_NETWORK={network}
HL_ACCOUNT_ADDRESS={account_address}
HL_SECRET_KEY={secret_key}
"""
    if api_secret_key:
        env_content += f"HL_API_SECRET_KEY={api_secret_key}\n"
    env_content += f"HL_LOG_LEVEL={log_level}\n"
    env_content += f"HL_LANGUAGE={language}\n"

    env_path = Path(".env")
    with open(env_path, "w") as f:
        f.write(env_content)

    print(f"\nConfiguration saved to {env_path.absolute()}")
    return config
